package Yapw::Format::Html;
use strict;
use Yapw::Format;
use vars qw(@ISA);

@ISA = qw(Yapw::Format);

my $DEBUG = "";

sub new{
    my $class = shift;
    my $new = new Yapw::Format;

    $new->{'root'} = shift;
    $new->{'SELFNAME'} = shift;

    return bless $new,$class;
}

sub pre_convert{
    my $self = shift;
    my($content) = shift;

    $content = $self->proofmark($content);
    return $content;
}

sub post_convert{
    my $self = shift;
    my($content) = (@_);

    $content = $self->heading($content);
    $content = $self->table($content);
    $content = $self->list($content);
#    $content = $self->to_html($content);

    if($DEBUG){
	$content = $content."{DEBUG: $DEBUG}\n";
	$DEBUG = "";
    }
    return $content;
}

sub eval_without_pre{
    my($func,$text) = @_;
    my $result;
    foreach(split(m{(<pre>.*?</pre>)}s,$text)){
	if(/^<pre>/){
	    $result .= $_;
	}else{
	    $result .= &$func($_);
	}
    }
    return $result;
}

# convert methods
sub proofmark{
    shift;
    return eval_without_pre(\&_proofmark,@_);
}

sub _proofmark{
    my($text) = @_;
    
    $text =~ s{([^\\])\[(.*?)>>(.*?)\]}{$1<del>$2</del><ins>$3</ins>}mg;
    return $text;
}

sub heading{
    shift;
    return eval_without_pre(\&_heading,@_);
}

sub _heading{
    my($text) = @_;
    
    my $lv;
    $text =~ s/^(\@+)(.*)$/
	$lv = length($1) + 1;sprintf("<h%d>%s<\/h%d>\n",$lv,$2,$lv)/mgex;

    return $text;
}

sub list{
    shift;
    return eval_without_pre(\&_list,@_);
}

sub _list_unshift_level{
    my($indent,$level,$mode) = @_;
    my($result,$lv,$ty);
    while(@$level){
	$lv = $$level[0]->{level};
	$ty = $$level[0]->{type};
	last unless ($indent < $lv);
	$result .= ' ' x ($lv - 1) . "</li></$ty>\n" if($$mode);
	shift(@$level);
    }
    $mode = 0 if($lv eq '');
    return $result;
}

sub _list{
    my($text) = @_;

    my($result);
    my(@level);
    my($mode) = 0;
    foreach(split(/\n/,$text)){
	/^((?:\s|)*)(.*?)$/;
	my $level     = length($1);
	my $indent    = ' ' x $level;
	my $space     = $1;
	my $content   = $2;
	my $list_mark = 0;
	do{$level++; 
	   $list_mark++;} if($content =~ s/^\*/<li>/);
	$list_mark++ if($content =~ s/^<li>\#/<li>/);
	
	my $current = $level[0];
	if($level < $current->{level}){
	    $result .= _list_unshift_level($level,\@level,\$mode);
	    $result .= '</li>' if($mode && $list_mark);
	}elsif($level == $current->{level}){
	    $result .= '</li>' if($mode && $list_mark);
	}else{# if($indent > $current->{level});
	    if($list_mark){
		my $type = ('','ul','ol')[$list_mark];
		unshift(@level,{ level => $level,
				 type  => $type       });
		$result .= ' ' x ($level - 1) . "<$type>\n";
	    }
	}
	$result .= $space.$content."\n";
	$mode |= $list_mark;
	$mode = 0 if($level[0] eq '');
    }
    $result .= _list_unshift_level(0,\@level,\$mode);
    $result =~ s{\n(\s*)</li>}{</li>\n$1}g;
    return $result;
}

sub table{
    shift;
    return eval_without_pre(\&_table,@_);
}

sub _table{
    my($text) = @_;
    my $is_table = 0;
    my $esc_line = 0;
    my($table,$result,$table_opt);
    foreach(split(m{\n}s,$text)){
	if($esc_line){
	    if(s/\\$//s){
		$esc_line = 1;
		$table .= "$_\\n";
	    }else{
		$esc_line = 0;
		$table .= "$_\n";
	    }
	    next;
	}
	if(/^\|\|(.*?)\|\|/s){
	    $is_table = 1;
	    $table_opt = $1;
	    next;
	}elsif(s/^\|//s){
	    $is_table = 1;
	    if(s/\\$//s){
		$esc_line = 1;
		$table .= "$_\\n";
	    }else{
		$table .= "$_\n";
	    }
	    next;
	}else{
	    if($table){
		$is_table = 0;
		$result .=  _table_maketable($table_opt,$table);
		$table = ""; $table_opt = "";
	    }
	    $result .= "$_\n";
	}
    }
    $result .= _table_maketable($table_opt,$table) if($table);
    $result =~ s/\\n/\n/g;
    return $result;
}

sub _table_maketable{
    my($table_opt,$text) = @_;
    my($result);
    $table_opt = " ".$table_opt if($table_opt);
    foreach(split("\n",$text)){
	my($line,@line);
	@line = split(/\|/,$_);
	while(@line){
	    $_ = shift(@line);
	    while(s/\\$//){
		$_ .=  "|". shift(@line);
	    }
	    my($colspan);
	    if(s/^(\s*)(~+)/$1/){
		$colspan = " colspan='".(length($2) + 1)."'";
	    }
	    $line .= "<td$colspan>$_</td>";
	}
	$result .= "<tr>$line</tr>\n" if($line);
    }
    $result = "<table$table_opt>\n$result</table>\n" if($result);
    return $result;
}

sub to_html{
    my $self = shift;
    return $self->eval_without_pre(\&_to_html_core,@_);
}

sub _to_html_core{
    my $self = shift;
    local($_) = @_;
    my($result,$elem,@paragraph,@temp,@elem);

    @paragraph = split(m{(</?(?:$self->PATTERN('NonEmpty'))(?: .*?)?>)}soi,$_);
#    foreach(@paragraph){
#	$DEBUG .= "$_\n";
#    }
    while(@paragraph){
	$_ = shift(@paragraph);
	if(m{<($self->PATTERN('Block'))( .*?)?>}oi){
	    my($elem) = $1;
	    push(@temp,$_);
	    while(@paragraph){
		$_ = shift(@paragraph);
		push(@temp,$_);
		last if(m{^<\/($elem)( .*?)?>}i);
	    }
	}else{
	    my $par;
	    while(@paragraph){
		if(m{</?($self->PATTERN('Block'))( .*?)?>}oi){
		    $par .= "\n\n$_";
		    $_ = "";
		    last;
		}
		$par .= $_;
		$_ = shift(@paragraph);
	    }
	    $par .= $_;
	    push(@temp,split(/(?:\s*?\n){2,}/s,$par));
	}
    } @paragraph = @temp;

    while(@paragraph){
	$_ = shift(@paragraph);
	if(m{</($self->PATTERN('Block'))>}oi){
	    my($elem) = $1;
	    while(@elem && $elem[0] ne $elem){
		shift(@elem);
	    }
	    shift(@elem);
	}elsif(m{^<($self->PATTERN('Block'))( .*?)?>}oi){
	    unshift(@elem,$1);
	}elsif(!@elem){
	    $_ = "<p>$_</p>\n";
	    s{(?:$self->{conf}->{pattern}->{CR}){2,}}{</p>\n<p>}smg;
	}
	s{\n}{<br />\n}mg;
	s{<(/?)($self->PATTERN('Block')|$self->PATTERN('InlineBlock'))>(\n*)<br />}{<$1$2>$3}sog;
	my $ELEM = $self->PATTERN('Block')."|".$self->PATTERN('InlineBlock');
	s{<br />(\n*)\s*<(/?)($ELEM)>}{$1<$2$3>}sog;
	s/^\s*<br \/>$//mg;
	s{</tr><br />}{</tr>}mg;
	s{</p><br />}{</p>}g;
	s{<p></p>}{\n}g;
	$result .= $_;
    }
    return $result;
}

sub pattern_index{
    my $self = shift;
    my $target = shift;
    $target ||= $main::target;
    local($_);
    my(@context,$foo);
    # make context pattern
    if($main::NameSpaceMark){
	my $NameSpaceMark = $main::NameSpaceMark;
	foreach(split(/$NameSpaceMark/,$target)){
	    next unless($_);
	    if(length($foo) == 0){
		$foo = $_.$NameSpaceMark;
	    }else{
		$foo = $foo.$_.$NameSpaceMark;
	    }
	    unshift(@context,$foo);
	}
    }
    my $context = join('|',sort {length($b) <=> length($a)} @context);
    # make name-map.
    $self->{RealName} ||= {};
    foreach (sort {length($a) <=> length($b)} ($self->{root}->__())){
	if(/^($context)(.*)$/){
            $self->{RealName}->{$2} = $_ unless ($1.$2 eq $target);
        }
        $self->{RealName}->{$_} = $_;
    }
    my(@Index) = sort {length($b) <=> length($a)} keys %{$self->{RealName}};
    return join('|', map {$_ = quotemeta($_)} @Index);
}

sub realname{
    my $self = shift;
    my $name = shift;
    return $self->{RealName}->{$name};
}
1;
